package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.RtcRoomJoinHisDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import com.starbuds.server.service.task.entity.rtc.StatRtcRoomDTO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 用户进房记录 Mapper
 */
@Mapper
@Repository
public interface RtcRoomJoinHisDAO {

    /**
     * 插入数据
     */
    int insert(RtcRoomJoinHisDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<RtcRoomJoinHisDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(RtcRoomJoinHisDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    RtcRoomJoinHisDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    RtcRoomJoinHisDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcRoomJoinHisDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("hisId")
    Map<Long, RtcRoomJoinHisDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 获取活跃用户
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<StatRtcRoomDTO> selectRoomJoinHisByTime(@Param("startTime") Long startTime, @Param("endTime") Long endTime);

    /**
     * 最高同时在线
     * 用户停留时长
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<RtcRoomJoinHisDO> selectRoomUserJoinHisByTime(@Param("startTime") Long startTime, @Param("endTime") Long endTime);

}