package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.service.task.entity.StatPlatformGatherDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Mapper
@Repository
public interface StatPlatformGatherDAO {

    /***
     * 获取指定支付类型的所有giftId
     * @param giftPriceType
     * @return
     */
    @MapKey("giftId")
    Map<Long, GiftDO> selectGiftId(@Param("giftPriceType") Long giftPriceType);

    /***
     * 获取所有触发活动ID
     * @return
     */
    List<Long> selectTriggerId();

    /***
     * 查询活动触发的礼物赠送
     * @param startTimestamp 大于等于 开始时间
     * @param endTimestamp 小于 结束时间
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectTriggerGiveGift(@Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp, @Param("cursor") int cursor, @Param("pageSize") int pageSize);

    /***
     * 查询触发的赠送金币
     * @param walletType
     * @param startTimestamp 大于等于 开始时间
     * @param endTimestamp 小于 结束时间
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectTriggerGiveCoin(@Param("walletType") Long walletType,
                                                     @Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp,
                                                     @Param("cursor") int cursor, @Param("pageSize") int pageSize);

    /***
     * 获取管理员 派币/扣币 金额
     * @param walletType
     * @param isAdd 1：派币  0：扣币
     * @param startTimestamp 大于等于 开始时间
     * @param endTimestamp 小于 结束时间
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectManageGiveCoin(@Param("walletType") Long walletType, @Param("isAdd") Long isAdd,
                                                    @Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp,
                                                    @Param("cursor") int cursor, @Param("pageSize") int pageSize);

    /***
     * 查询区间范围的总值
     * @param type
     * @param scale
     * @param ext
     * @param startTimestamp
     * @param endTimestamp
     * @return
     */
    Long selectSumQuantity(@Param("type") Long type, @Param("scale") Long scale, @Param("ext") Long ext,
                           @Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp);

    /***
     * 当前汇总条件是否存在记录
     * @param type
     * @param scale
     * @param timestamp
     * @return
     */
    int countExistGatherRecord(@Param("type") Long type, @Param("scale") Long scale, @Param("timestamp") Long timestamp, @Param("ext") Long ext);

    /***
     * 更新汇总合算值
     * @param quantity
     * @param type
     * @param scale
     * @param timestamp
     * @return
     */
    int updateGatherQuantity(@Param("quantity") Long quantity, @Param("type") Long type, @Param("scale") Long scale, @Param("timestamp") Long timestamp, @Param("ext") Long ext);

    /***
     * 批量插入或更新数据
     * @param list SQL语句列表
     * @return
     */
    int insertBatch(List<String> list);

    /***
     * 擦除时间段为小时的数据记录
     * @param startTimestamp
     * @param endTimestamp
     * @param type
     * @param scale
     * @param ext
     * @return
     */
    int eraseScaleHourRecord(@Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp, @Param("type") Long type, @Param("scale") Long scale, @Param("ext") Long ext);


    /***
     * 查询活动触发的礼物赠送
     * @param giftPriceType
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectTriggerGiveGift1(@Param("giftPriceType") Long giftPriceType, @Param("timestamp") Long timestamp, @Param("cursor") int cursor, @Param("pageSize") int pageSize);

    /***
     * 查询触发的赠送金币
     * @param walletType
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectTriggerGiveCoin1(@Param("walletType") Long walletType, @Param("timestamp") Long timestamp, @Param("cursor") int cursor, @Param("pageSize") int pageSize);

    /***
     * 获取管理员 派币/扣币 金额
     * @param walletType
     * @param isAdd 1：派币  0：扣币
     * @param timestamp
     * @param cursor
     * @param pageSize
     * @return
     */
    List<StatPlatformGatherDO> selectManageGiveCoin1(@Param("walletType") Long walletType, @Param("isAdd") Long isAdd, @Param("timestamp") Long timestamp, @Param("cursor") int cursor, @Param("pageSize") int pageSize);


}
