package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.UserAuthDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 用户认证信息持久层
 *
 * @author aoe
 * @date 2021/8/26
 */
@Mapper
@Repository
public interface UserAuthDAO {

    /**
     * 获取用户授权
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, UserAuthDO> selectAuthMapByIds(@Param("userIds") Set<Long> userIds);

    /**
     * 批量插入认证记录数据
     *
     * @param authDOList
     * @return
     */
    int insertOrUpdateUserAuthHisBatch(List<UserAuthDO> authDOList);

    /**
     * 批量查询用户认证信息
     *
     * @param pageIndex
     * @param pageSize
     * @return
     */
    List<UserAuthDO> selectAuthList(@Param("cursor") int pageIndex, @Param("pageSize") int pageSize);

    /**
     * 删除UserAuth表中的未成年人记录
     *
     * @param authDOList 未成年认证信息
     * @return
     */
    int deleteUnderAgeAuthBatch(List<UserAuthDO> authDOList);
}
