package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.CloudAccountTransferHisDO;
import com.starbuds.server.common.pojo.db.EasyTaxTransferHisDO;
import com.starbuds.server.common.pojo.db.UserWalletWithdrawHisDO;
import com.starbuds.server.common.pojo.finance.ThirdPartyWithdrawHisDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 用户提现
 *
 * @author aoe
 * @date 2021/8/26
 */
@Mapper
@Repository
public interface UserWalletWithdrawHisDAO {

    /**
     * 获取提现列表
     *
     * @param pageCursor
     * @param pageSize
     * @param startTime
     * @param endTime
     * @param refType
     * @return
     */
    List<UserWalletWithdrawHisDO> getUserWalletWithdrawPageList(@Param("pageCursor") int pageCursor,
                                                                @Param("pageSize") int pageSize,
                                                                @Param("startTime") Long startTime,
                                                                @Param("endTime") Long endTime,
                                                                @Param("refType") Integer refType);

    /**
     * 根据状态获取提现列表
     *
     * @param pageCursor
     * @param pageSize
     * @param applyStatus
     * @param refType
     * @return
     */
    List<UserWalletWithdrawHisDO> getUserWalletWithdrawList(@Param("pageCursor") int pageCursor,
                                                            @Param("pageSize") int pageSize,
                                                            @Param("applyStatus") Integer applyStatus,
                                                            @Param("refType") Integer refType);

    /**
     * 获取提现列表总数
     *
     * @param startTime
     * @param endTime
     * @param refType
     * @return
     */
    Integer getUserWalletWithdrawPageListCount(@Param("startTime") Long startTime,
                                               @Param("endTime") Long endTime,
                                               @Param("refType") Integer refType);

    /**
     * 根据ids获得 云账户转账记录 Map
     *
     * @param ids 业务id集合
     * @return Map<业务id, 云账户转账记录>
     */
    @MapKey("bizId")
    Map<Long, CloudAccountTransferHisDO> selectCloudAccountTransferHisMapByBizIds(@Param("ids") Collection ids);

    /**
     * 根据ids获得 易纳税转账记录 Map
     *
     * @param ids 业务id集合
     * @return Map<业务id, 易纳税转账记录>
     */
    @MapKey("bizId")
    Map<Long, EasyTaxTransferHisDO> selectEasyTaxTransferHisMapByBizIds(@Param("ids") Collection ids);

    /**
     * 根据ids获得 第三方提现记录 Map
     *
     * @param ids 业务id集合
     * @return Map<业务id, 第三方提现记录>
     */
    @MapKey("userWalletWithdrawHisId")
    Map<Long, ThirdPartyWithdrawHisDO> selectThirdPartyWithdrawHisMapByBizIds(@Param("ids") Collection ids);
}
