package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.pojo.activity.bo.AnnualGalaPeakFamilyRankBO;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardConfigDO;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardItemDO;
import com.starbuds.server.common.pojo.db.UserActivityScoreDO;
import com.starbuds.server.common.pojo.message.im.activity.AnnualGalaFinishMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.activity.ActivityAnnualAwardConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author laoliu
 * @date 2021/12/14 15:37
 * @description 巅峰家族活动结束
 * 跑任务时，参数传入activityId
 */
@Component
@Slf4j
public class AnnualGalaPeakFamilyJob {

    @Reference(version = "1.0.0")
    ActivityAnnualAwardConfigService activityAnnualAwardConfigService;

    @Reference(version = "1.0.0")
    ActivityAnniversaryService activityAnniversaryService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @XxlJob(value = JobHandlers.ANNUAL_GALA_PEAK_FAMILY_FINISH)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> annualGalaPeakFamilyFinishHandler(String param) throws ParseException {

        if (param == null) {
            return new ReturnT<>("activityId必传");
        }

        Long activityId = Long.valueOf(param);
        log.info("巅峰家族活动结束: 活动activityId: {}", activityId);

        // 1排行榜
        AnnualGalaFinishMsg finishMsg = new AnnualGalaFinishMsg();
        finishMsg.setActivityType(ActivityTypeEnum.PEAKFAMILY.getValue());
        finishMsg.setMsgType(1);
        finishMsg.setActivityId(activityId);
        finishMsg.setTitle("巅峰家族Top5");
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, null, finishMsg);

        return new ReturnT<>("成功");
    }
}
