package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.pojo.db.UserAuthDO;
import com.starbuds.server.common.pojo.finance.UserBankcardDO;
import com.starbuds.server.common.utils.UserUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UserAuthDAO;
import com.starbuds.server.service.task.dao.UserBankcardDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/***
 * @author qinjiang
 * @description 审核用户身份认证(去除小于18岁的用户认证信息和银行卡)
 * @date 2021/9/8
 */
@Slf4j
@Component
public class CensorUserAuthorJob {

    @Autowired
    UserAuthDAO userAuthDAO;

    @Autowired
    UserBankcardDAO userBankcardDAO;

    @XxlJob(value = JobHandlers.CENSOR_USER_UNDERAGE_TASK)
    public ReturnT<String> censorUserUnderAgeHandler(String param) {
        int pageIndex = 1;
        int pageSize = 50000;
        long beginTimestamp = System.currentTimeMillis();
        boolean isRunloop = true;

        while (isRunloop) {
            List<UserAuthDO> userAuthDOList = userAuthDAO.selectAuthList((pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(userAuthDOList) || userAuthDOList.size() <= 0) {
                break;
            }
            if (userAuthDOList.size() < pageSize) {
                isRunloop = false;
            }

            List<UserAuthDO> pendingUserAuthList = new ArrayList<>();
            List<UserBankcardDO> pendingUserBankcardList = new ArrayList<>();
            for (UserAuthDO userAuthDO : userAuthDOList) {
                int userAge = UserUtil.getAgeCertID(userAuthDO.getIdCardNo());
                if (userAge < 18) {
                    pendingUserAuthList.add(userAuthDO);
                    List<UserBankcardDO> bankcardDOList = userBankcardDAO.selectUserbankList(userAuthDO.getUserId());
                    if (bankcardDOList.size() > 0) {
                        pendingUserBankcardList.addAll(bankcardDOList);
                    }
                }
            }

            if (pendingUserAuthList.size() > 0) {
                log.info("--CENSOR_USER_UNDERAGE_TASK-- 待处理用户：" + pendingUserAuthList.size());
                int insertAuthHisResult = userAuthDAO.insertOrUpdateUserAuthHisBatch(pendingUserAuthList);
                if (insertAuthHisResult > 0) {
                    userAuthDAO.deleteUnderAgeAuthBatch(pendingUserAuthList);
                }
            }

            if (pendingUserBankcardList.size() > 0) {
                log.info("--CENSOR_USER_UNDERAGE_TASK-- 待处理用户：" + pendingUserAuthList.size());
                int insertBankcardHisResult = userBankcardDAO.insertOrUpdateUserBankcardHisBatch(pendingUserBankcardList);
                if (insertBankcardHisResult > 0) {
                    userBankcardDAO.deleteUnderAgeBankcardBatch(pendingUserBankcardList);
                }
            }

            pageIndex++;
        }

        long endTimestamp = System.currentTimeMillis();
        log.info("--CENSOR_USER_UNDERAGE_TASK--审核用户身份认证任务完成，共耗时：" + (endTimestamp - beginTimestamp) + "ms");

        return ReturnT.SUCCESS;
    }
}
