package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import com.starbuds.server.common.pojo.db.FamilyUserDO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.FamilyAuditDAO;
import com.starbuds.server.service.task.dao.FamilySkillAuditDAO;
import com.starbuds.server.service.task.dao.FamilySkillDAO;
import com.starbuds.server.service.task.dao.FamilyUserDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/11/23 14:16
 * @Description 家族强制解约定时任务
 */
@Slf4j
@Component
public class FamilyAuditJob {

    @Resource
    FamilyAuditDAO familyAuditDAO;

    @Resource
    FamilyUserDAO familyUserDAO;

    @Resource
    FamilySkillDAO familySkillDAO;

    @Resource
    FamilySkillAuditDAO familySkillAuditDAO;

    /**
     * 家族申请
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.FAMILY_AUDIT)
    public ReturnT<String> familyAuditJobHandler(String param) {
        log.info("处理家族申请 强制解约 {} 当前时间 {}", param, DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
        // 待审核
        List<Long> auditIds = familyAuditDAO.selectAuditIds(Convert.toLong(DateUtil.format(new Date(), "yyyyMMdd")));
        log.info("auditIds {}", auditIds);
        if (CollectionUtil.isNotEmpty(auditIds)) {
            for (Long auditId : auditIds) {
                try {
                    long curTime = System.currentTimeMillis();
                    FamilyAuditDO auditDO = familyAuditDAO.selectById(auditId);
                    if (auditDO == null) {
                        continue;
                    }
                    if (AuditStatusEnum.WAIT.getCode() != auditDO.getAuditStatus()) {
                        continue;
                    }
                    FamilyUserDO userDO = new FamilyUserDO();
                    userDO.setUserId(auditDO.getUserId());
                    userDO.setFamilyId(auditDO.getFamilyId());
                    userDO.setSignTime(null);
                    userDO.setEndTime(null);
                    userDO.setStatus(CommonStatus.Deleted);
                    userDO.setRatio(null);
                    familyUserDAO.insertOrUpdate(userDO);
                    // 处理申请
                    familyAuditDAO.updateStatus(auditId, AuditStatusEnum.PASS.getCode(), "强制解约生效", curTime, 0L);
                    // 作废技能
                    familySkillDAO.updateSkillStatus(auditDO.getFamilyId(), auditDO.getUserId(), CommonStatus.Deleted, curTime);
                    familySkillAuditDAO.updateStatusByUserId(auditDO.getFamilyId(), auditDO.getUserId(), AuditStatusEnum.DENY.getCode(), "强制解约生效", 0L, curTime);
                } catch (Exception e) {
                    log.info("出现异常 {}", auditId, e);
                }
            }
        }
        return ReturnT.SUCCESS;
    }

}
