package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityGrabPeriodStatusEnum;
import com.starbuds.server.common.pojo.db.ActivityGrabConfigDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodDO;
import com.starbuds.server.common.service.activity.GrabActivityService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ActivityGrabPeriodDAO;
import com.starbuds.server.service.task.dao.ActivityGrabPeriodItemDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2021/4/4
 */
@Component
@Slf4j
public class GrabJob {

    @Resource
    ActivityGrabPeriodDAO activityGrabPeriodDAO;

    @Resource
    ActivityGrabPeriodItemDAO activityGrabPeriodItemDAO;

    @Reference(version = "1.0.0")
    GrabActivityService grabActivityService;

    @XxlJob(value = JobHandlers.GRAB_SETTLE)
    public ReturnT<String> grabSettle(String param) {
        Long periodId;
        if (StringUtils.isNotBlank(param)) {
            periodId = Convert.toLong(param);
        } else {
            // 查询最新一期
            periodId = activityGrabPeriodDAO.selectLatestPeriodId();
        }
        log.info("Grab 开始 periodId = {}", periodId);

        if (periodId != null && periodId > 0) {
            // 判断状态
            ActivityGrabPeriodDO period = activityGrabPeriodDAO.selectPeriodStatus(periodId);
            if (period.getPeriodStatus() != null && ActivityGrabPeriodStatusEnum.Going.getValue() == period.getPeriodStatus()) {
                ActivityGrabConfigDO config = JSON.parseObject(period.getConfigSnapshot(), ActivityGrabConfigDO.class);
                if (config != null) {
                    // 判断参与人数
                    List<Long> userIds = activityGrabPeriodItemDAO.selectUserIds(periodId);
                    if (userIds.size() == config.getPlayerNum()) {
                        // 结算
                        grabActivityService.settle(periodId, userIds);
                    }
                }
            }
        }
        return ReturnT.SUCCESS;
    }
}
