package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.media.StreamStatusEnum;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.LiveHisDO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.AnchorDAO;
import com.starbuds.server.service.task.dao.LiveHisDAO;
import com.starbuds.server.service.task.dao.LiveStreamHisDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 直播监管任务
 *
 * @author Lay
 * @date 2020/2/24
 */
@Component
@Slf4j
public class LiveMonitorJob {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Resource
    AnchorDAO anchorDAO;

    @Resource
    LiveHisDAO liveHisDAO;

    @Resource
    LiveStreamHisDAO liveStreamHisDAO;

    /**
     * 直播监控状态处理
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.LIVE_MONITOR_STATUS)
    public ReturnT<String> statusJobHandler(String param) {
        log.info("直播监控状态处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();

        // 找出所有在线主播
        Set<Tuple> lives = cacheService.zRangeWithScores(RedisLiveKeys.LIVE_POPULARITY_SET, 0, -1);

        List<Long> anchorIds = new ArrayList<>();
        int i = 0;
        for (Tuple tuple : lives) {
            if (shardingVO.getTotal() == 1 || i % shardingVO.getTotal() == shardingVO.getIndex()) {
                // 命中分片
                anchorIds.add(Convert.toLong(tuple.getElement()));
            }
            i++;
        }

        if (CollectionUtil.isNotEmpty(anchorIds)) {
            List<Long> ids = anchorDAO.selectIdsByStatus(anchorIds, LiveStatusEnum.Living.getValue(), StreamStatusEnum.Closed.getValue());
            if (CollectionUtil.isNotEmpty(ids)) {
                for (Long anchorId : ids) {
                    // 查询最后一条直播记录
                    LiveHisDO liveHisDO = liveHisDAO.selectLastByUserId(anchorId, LiveStatusEnum.Living.getValue());
                    if (liveHisDO != null && liveHisDO.getLiveStartTime() != null && liveHisDO.getLiveStopTime() == null) {
                        // 判断开播时间是否超过 10 分钟，且没有推流记录
                        Long timeout = System.currentTimeMillis() - liveHisDO.getLiveStartTime();
                        if (liveStreamHisDAO.countByLiveHisId(liveHisDO.getHisId()) == 0 && timeout > 600 * 1000L) {
                            // 超时关播
                            log.info("主播 {} 因超时 {} 毫秒被执行超时处理", anchorId, timeout);
                            liveService.stopLive(liveHisDO.getHisId(), true);
                        }
                    }
                }
            }
        }

        return ReturnT.SUCCESS;
    }
}
