package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.data.BagItem;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.PropsDO;
import com.starbuds.server.common.pojo.db.UserBagDO;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.GiftDAO;
import com.starbuds.server.service.task.dao.PropsDAO;
import com.starbuds.server.service.task.dao.UserBagDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 道具回收
 *
 * @author Lay
 * @date 2021/10/9
 */
@Component
@Slf4j
public class PropsRecycleJob {

    @Resource
    PropsDAO propsDAO;

    @Resource
    GiftDAO giftDAO;

    @Resource
    UserBagDAO userBagDAO;

    @Reference(version = "1.0.0")
    BagService bagService;

    @XxlJob(value = JobHandlers.PROPS_RECYCLE_GIFT_FRAGMENT)
    public ReturnT<String> propsRecycleGiftFragment(String param) {
        log.info("礼物碎片回收 {}", param);
        Long propsId = Convert.toLong(param, 0L);
        if (propsId < 1) {
            log.warn("无效参数，任务中止");
            return ReturnT.SUCCESS;
        }

        // 查询碎片信息
        PropsDO props = propsDAO.selectById(propsId);
        if (props == null || PropsTypeEnum.GIFT_FRAGMENT.getValue() != props.getPropsType()) {
            log.warn("非有效礼物碎片，任务中止");
            return ReturnT.SUCCESS;
        }

        // 查询礼物信息
        GiftDO gift = giftDAO.selectById(Convert.toLong(props.getPropsExtString()));
        if (gift == null) {
            log.warn("未找到碎片对应的礼物，任务中止");
            return ReturnT.SUCCESS;
        }

        // 开始处理
        long fragmentPrice = gift.getGiftPrice() / props.getPropsExtInt();
        log.info("开始处理礼物碎片:{}, 对应礼物: {}, 礼物价值: {}, 合成需要: {}, 碎片价值: {}", props.getPropsName(), gift.getGiftName(), gift.getGiftPrice(), props.getPropsExtInt(), fragmentPrice);

        // 查询背包
        List<UserBagDO> bagItemList = userBagDAO.selectByItemId(propsId);
        if (bagItemList.size() == 0) {
            log.warn("用户背包中无此碎片数据，任务中止");
            return ReturnT.SUCCESS;
        }

        int quantities = 0;
        for (UserBagDO userBag : bagItemList) {
            // 开始处理
            if (userBag.getQuantity() == 0) {
                continue;
            }

            // 移除
            quantities += userBag.getQuantity();
            bagService.takeItem(userBag.getUserId(), propsId, BagItemTypeEnum.PROPS.getValue(), userBag.getQuantity(), "碎片回收", 0L);
        }

        log.info("处理完成，共处理碎片 {} 个，价值 {}", quantities, quantities * fragmentPrice);
        return ReturnT.SUCCESS;
    }
}
