package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.UserRechargeDailyDO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.service.task.async.RechargeDailyPersistenceAsync;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UserRechargeDailyDAO;
import com.starbuds.server.service.task.entity.ShardingParam;
import com.starbuds.server.service.task.util.ShardingHelper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 每日充值持久化任务
 *
 * @author Lay
 * @date 2020/2/28
 */
@Component
@Slf4j
public class RechargeDailyPersistenceJob {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Resource
    UserRechargeDailyDAO userRechargeDailyDAO;

    @Resource
    RechargeDailyPersistenceAsync rechargeDailyPersistenceAsync;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.RECHARGE_DAILY)
    public ReturnT<String> jobHandler(String param) {
        log.info("触发每日充值持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 查询日期处理
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer day;
        if (StringUtils.isNotBlank(param)) {
            // 有传参取传参的
            day = Convert.toInt(param);
        } else {
            // 不然就是取前一天的，因为定时任务一定是后一天来统计前一天的
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -1);
            day = Convert.toInt(sdf.format(calendar.getTime()));
        }
        if (day != null) {
            String key = RedisFinanceKeys.getRechargeDaily(day);
            // 取出排行数据
            Long size = cacheService.zCard(key);
            ShardingParam shardingParam = ShardingHelper.getParam(shardingVO, size);
            if (shardingParam != null) {
                Set<Tuple> tupleSet = cacheService.zReverseRangeWithScores(key, shardingParam.getStart(), shardingParam.getEnd());
                if (tupleSet != null) {
                    List<Tuple> list = new ArrayList<>(tupleSet);
                    list.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                    // 执行处理
                    List<UserRechargeDailyDO> userRechargeDailyList = new ArrayList<>();
                    int rankNo = shardingParam.getStart() + 1;
                    Long curTime = System.currentTimeMillis();
                    for (Iterator<Tuple> iterator = list.iterator(); iterator.hasNext(); ) {
                        Tuple tuple = iterator.next();
                        UserRechargeDailyDO userRechargeDaily = new UserRechargeDailyDO();
                        userRechargeDaily.setDateNum(day);
                        userRechargeDaily.setUserId(Convert.toLong(tuple.getElement()));
                        userRechargeDaily.setAmount(new BigDecimal(String.valueOf(tuple.getScore())));
                        userRechargeDaily.setRankNo(rankNo);
                        userRechargeDaily.setUpdateTime(curTime);

                        userRechargeDailyList.add(userRechargeDaily);
                        rankNo++;
                    }

                    // 保存
                    userRechargeDailyDAO.batchInsert(userRechargeDailyList);

                    if (shardingParam.getEnd() == -1) {
                        rechargeDailyPersistenceAsync.checkSize(30 * 1000L, size, day, key);
                    }
                }
            }
            return ReturnT.SUCCESS;
        }
        return ReturnT.FAIL;
    }
}
