package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.task.ExportExcelBizTypeEnum;
import com.starbuds.server.common.constant.task.ExportExcelDimensionEnum;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ExcelExportDAO;
import com.starbuds.server.service.task.dao.FamilyWalletHisDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.starbuds.server.service.task.dao.UserWalletHisDAO;
import com.starbuds.server.service.task.entity.GiftGratuityExcel;
import com.starbuds.server.service.task.provider.ExcelProvider;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/***
 * @author qinjiang
 * @description （直播人员打赏记录）用户打赏礼物报表
 * @date 2021/8/25
 */
@Slf4j
@Component
public class ReportGiftGratuityJob {

    @Resource
    UserDAO userDAO;

    @Resource
    UserWalletHisDAO userWalletHisDAO;

    @Resource
    FamilyWalletHisDAO familyWalletHisDAO;

    @Resource
    ExcelExportDAO excelExportDAO;

    @Resource
    ExcelProvider excelProvider;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    private static final int pageSize = 50000;
    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.SIMPLIFIED_CHINESE);

    @XxlJob(value = JobHandlers.REPORT_GIFT_GRATUITY_TASK)
    public ReturnT<String> giftGratuityReport(String param) {
        /**
         *  {"idCardNo":["44030119980217411X","2303011998021755410","440301199802174119"],"startTimestamp":1627747200000,"endTimestamp":1630425600000,"filter":0,"type":22}
         */

        if (StringUtils.isBlank(param)) {
            return ReturnT.FAIL;
        }

        JSONObject jsonObject = JSON.parseObject(param);
        if (jsonObject == null) {
            return ReturnT.FAIL;
        }

        Long startTimestamp = jsonObject.getLong("startTimestamp");
        Long endTimestamp = jsonObject.getLong("endTimestamp");
        Long type = jsonObject.getLong("type");
        Long filter = jsonObject.getLong("filter");

        Iterator<Object> idCardNoIterator = jsonObject.getJSONArray("idCardNo").iterator();
        while (idCardNoIterator.hasNext()) {
            String idCardNo = (String) idCardNoIterator.next();
            List<Long> listUserID = userDAO.getUserIDByIDCardNo(idCardNo);
            for (Long userID : listUserID) {
                UserDO userDO = userDAO.getUserByUserId(userID);
                if (userDO != null) {
                    reportUserWalletHisHander(userID, userDO.getUserNo(), userDO.getUserName(), idCardNo, type, filter.intValue(), startTimestamp, endTimestamp);
                }
            }

            //先查实名用户是否存在，再查是否为公会长
            List<FamilyBankcardDO> familyBankcardDOs = familyWalletHisDAO.getFamilyBankcard(idCardNo);
            if (!CollectionUtils.isEmpty(familyBankcardDOs)) {
                for (FamilyBankcardDO familyBankcardDO : familyBankcardDOs) {
                    reportFamilyWalletHisHander(familyBankcardDO.getFamilyId(), familyBankcardDO.getCardName(), familyBankcardDO.getCardIdCard(), startTimestamp, endTimestamp);
                }
            }
        }

        return ReturnT.SUCCESS;
    }

    /**
     * 查询用户钱包记录
     *
     * @param userId
     * @param userNo
     * @param userName
     * @param idCardNo
     * @param type
     * @param filter
     * @param startTimestamp
     * @param endTimestamp
     */
    protected void reportUserWalletHisHander(Long userId, Long userNo, String userName, String idCardNo, Long type, int filter, Long startTimestamp, Long endTimestamp) {
        int pageIndex = 1;
        boolean isRunloop = true;
        long useTimeStart = System.currentTimeMillis();

        List<GiftGratuityExcel> giftGratuityExcels = new ArrayList<>();
        while (isRunloop) {
            List<UserWalletHisDO> userWalletHisDOList = userWalletHisDAO.getUserWalletHis(userId, type, filter, startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(userWalletHisDOList) || userWalletHisDOList.size() <= 0) {
                break;
            }
            if (userWalletHisDOList.size() < pageSize) {
                isRunloop = false;
            }

            for (UserWalletHisDO hisDo : userWalletHisDOList) {
                String zoneTime = simpleDateFormat.format(new Date(hisDo.getCreateTime()));

                GiftGratuityExcel excel = new GiftGratuityExcel();
                excel.setUserId(userId.toString());
                excel.setUserNo(userNo.toString());
                excel.setUserName(userName);
                excel.setIdCardNo(idCardNo);
                excel.setRemark(hisDo.getHisRemarks());
                excel.setBalanceValue(hisDo.getBalanceValue().toString());
                excel.setTimestamp(hisDo.getCreateTime().toString());
                excel.setSendTime(zoneTime);

                giftGratuityExcels.add(excel);
            }

            pageIndex++;
        }

        log.info("REPORT_GIFT_GRATUITY_TASK - reportUserWalletHisHander :" + giftGratuityExcels.size());
        if (giftGratuityExcels.size() > 0) {
            String fileName = String.format("%s_%s.xlsx", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")), idCardNo);
            excelProvider.save2Disk(fileName, userName, giftGratuityExcels, GiftGratuityExcel.class);

            useTimeStart = System.currentTimeMillis() - useTimeStart;
            String remark = String.format("UserNo：%d  证件号码：%s", userNo, idCardNo);
            saveRecord(excelProvider.getDir() + fileName, fileName, startTimestamp, endTimestamp, (int) useTimeStart, remark, giftGratuityExcels.size());
        }
    }

    /**
     * 查询公会长钱包记录
     *
     * @param familyID
     * @param familyName
     * @param idCardNo
     * @param startTimestamp
     * @param endTimestamp
     */
    protected void reportFamilyWalletHisHander(Long familyID, String familyName, String idCardNo, Long startTimestamp, Long endTimestamp) {
        int pageIndex = 1;
        boolean isRunloop = true;
        long useTimeStart = System.currentTimeMillis();

        List<GiftGratuityExcel> giftGratuityExcels = new ArrayList<>();
        while (isRunloop) {
            List<FamilyWalletHisDO> familyWalletHisDOList = familyWalletHisDAO.getFamilyWalletHis(familyID, startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(familyWalletHisDOList) || familyWalletHisDOList.size() <= 0) {
                break;
            }
            if (familyWalletHisDOList.size() < pageSize) {
                isRunloop = false;
            }

            for (FamilyWalletHisDO hisDo : familyWalletHisDOList) {
                String zoneTime = simpleDateFormat.format(new Date(hisDo.getCreateTime()));

                GiftGratuityExcel excel = new GiftGratuityExcel();
                excel.setUserId(familyID.toString());
                excel.setUserNo("");
                excel.setUserName(familyName);
                excel.setIdCardNo(idCardNo);
                excel.setRemark(hisDo.getHisRemarks());
                excel.setBalanceValue(hisDo.getBalanceValue().toString());
                excel.setTimestamp(hisDo.getCreateTime().toString());
                excel.setSendTime(zoneTime);

                giftGratuityExcels.add(excel);
            }

            pageIndex++;
        }

        log.info("REPORT_GIFT_GRATUITY_TASK - reportFamilyWalletHisHander :" + giftGratuityExcels.size());
        if (giftGratuityExcels.size() > 0) {
            String fileName = String.format("%s_%s.xlsx", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")), idCardNo);
            excelProvider.save2Disk(fileName, familyName, giftGratuityExcels, GiftGratuityExcel.class);

            useTimeStart = System.currentTimeMillis() - useTimeStart;
            String remark = String.format("公会ID：%s  证件号码：%s", familyID, idCardNo);
            saveRecord(excelProvider.getDir() + fileName, fileName, startTimestamp, endTimestamp, (int) useTimeStart, remark, giftGratuityExcels.size());
        }
    }

    protected boolean saveRecord(String filePath, String filename, Long startTimestamp, Long endTimestamp, int useTimeStart, String remark, long sizeCount) {
        ExcelExportDO exportDO = new ExcelExportDO();
        exportDO.setExcelId(identityService.getId());
        exportDO.setExcelName(filename);
        exportDO.setBizName(ExportExcelBizTypeEnum.GIFT_GRATUITY.getLabel());
        exportDO.setBizType(ExportExcelBizTypeEnum.GIFT_GRATUITY.getValue());
        exportDO.setDate(DateUtil.dateToLocalDate(new Date(startTimestamp)).getDayOfMonth());
        exportDO.setMonth(DateUtil.dateToLocalDate(new Date(startTimestamp)).getMonthValue());
        exportDO.setYear(DateUtil.dateToLocalDate(new Date(startTimestamp)).getYear());
        exportDO.setCreateTime(System.currentTimeMillis());
        exportDO.setUpdateTime(System.currentTimeMillis());
        exportDO.setPath(filePath);
        exportDO.setFileSize(FileUtil.size(new File(filePath)));
        exportDO.setDimension(ExportExcelDimensionEnum.CUSTOMIZE.getValue());
        exportDO.setRemark(remark);
        exportDO.setRowCount(sizeCount);
        exportDO.setUseTime(useTimeStart);

        return excelExportDAO.insert(exportDO) > 0;
    }
}
