package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.pojo.rtc.RtcRoomHeat;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static com.xxl.job.core.biz.model.ReturnT.FAIL_CODE;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/24 11:26
 * @Description 语音房mvp改变
 */
@Slf4j
@Component
public class RtcMvpChangeJob {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    RtcRankService rtcRankService;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.RTC_MVP_CHANGE)
    public ReturnT<String> rtcMvpChangeHandler(String param) {
        log.info("执行语音房mvp改变 {}", param);
        try {
            List<Long> roomIds = new ArrayList<>();
            // 入参房间
            if (StringUtils.isNotEmpty(param)) {
                roomIds = Arrays.stream(param.split(",")).map(Convert::toLong).collect(Collectors.toList());
            }
            if (CollectionUtil.isEmpty(roomIds)) {
                roomIds = redisService.zRange(RedisRtcKeys.RTC_ROOM_HEAT, 0, -1).stream().map(Convert::toLong).collect(Collectors.toList());
            }

            rtcRankService.sendRtcMvpChangeMsg(roomIds);

            log.info("执行成功");
            return ReturnT.SUCCESS;
        } catch (Exception e) {
            log.error("执行出错", e);
            return new ReturnT(FAIL_CODE, e.getMessage());
        }
    }

    /**
     * 获取房间热度
     *
     * @return
     */
    private List<RtcRoomHeat> getRoomHeat(List<Long> roomIds) {

        if (CollectionUtil.isEmpty(roomIds)) {
            return new ArrayList<>();
        }

        List<RtcRoomHeat> liveHeats = new ArrayList<>();
        for (Long roomId : roomIds) {
            RtcRoomHeat liveHeat = new RtcRoomHeat();
            liveHeat.setRoomId(roomId);
            Set<Object> userIds = redisService.hashKeys(RedisRtcKeys.getRtcRoomUser(liveHeat.getRoomId()));
            liveHeat.setHeatValue(CollectionUtil.isNotEmpty(userIds) ? 100L * userIds.size() : 0L);
            liveHeats.add(liveHeat);
        }
        return liveHeats;
    }

}
