package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.RtcRoomLiveHisDO;
import com.starbuds.server.common.pojo.db.RtcRoomLiveOfflineHisDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.rtc.RtcRoomLiveHisService;
import com.starbuds.server.common.service.rtc.RtcRoomLiveOfflineHisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.RtcRoomDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static com.starbuds.server.service.task.util.SpringContextUtil.isDev;

/**
 * @Author laoliu
 * @DateTime 2021/10/29 11:26
 * @Description 个播主播离线过5分钟就关播
 */
@Slf4j
@Component
public class RtcSingleLiveOfflineJob {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Reference(version = "1.0.0", timeout = 5000)
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0", timeout = 5000)
    RtcRoomLiveOfflineHisService rtcRoomLiveOfflineHisService;

    @Reference(version = "1.0.0", timeout = 5000)
    RtcRoomLiveHisService rtcRoomLiveHisService;

    @Reference(version = "1.0.0")
    UserService userService;

    // 5分钟
    private static Long fiveMinutes = 5 * 60 * 1000L;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.RTC_SINGLE_LIVE_OFFLINE)
    public ReturnT<String> rtcSingleLiveOfflineHandler(String param) {
        log.info("执行个播主播离线关播检测开始 {}", param);
        Long now = new Date().getTime();

        List<RtcRoomLiveOfflineHisDO> list = rtcRoomLiveOfflineHisService.getUnDealRecord();

        if (CollectionUtil.isEmpty(list)) {
            log.info("执行个播主播离线关播检测结束, 未发现离线主播");
            return ReturnT.SUCCESS;
        }

        if (isDev()) {
            log.info("发现个播主播离线关播检测:{}", JSON.toJSONString(list));
        }

        try {

            for (int i = 0; i < list.size(); i++) {
                RtcRoomLiveOfflineHisDO val = list.get(i);

                // 判断用户在线跟房间
                RtcRoomListItem room = rtcRoomService.getUserStayRoom(val.getUserId());

                if (isDev()) {
                    log.info("i: {}, 判断用户在线跟房间:{}", i, JSON.toJSONString(room));
                }

                if (userService.isOnline(val.getUserId())
                        && room != null
                        && room.getRoomId().equals(val.getRoomId())) {
                    // 用户已经为在线状态了
                    log.info("执行个播主播离线关播检测结束-用户已重新上线:{}", JSON.toJSONString(val));
                    rtcRoomLiveOfflineHisService.setUnDealRecord(val.getHisId());
                    continue;
                }

                if (isDev()) {
                    log.info("i: {}, 判断离开时间是否已经过了5分钟:{}", i, now >= val.getLeaveTime() + fiveMinutes);
                }

                // 判断离开时间是否已经过了5分钟
                if (now >= val.getLeaveTime() + fiveMinutes) {

                    RtcRoomLiveHisDO currentRound = rtcRoomLiveHisService.getCurrentRound(val.getRoomId());
                    if (currentRound == null) {
                        log.info("执行个播主播离线关播检测开始 当前无直播场次，跳过处理:offline:{}", JSON.toJSONString(val));
                        rtcRoomLiveOfflineHisService.setUnDealRecord(val.getHisId());
                        continue;
                    }
                    if (currentRound.getHisId().equals(val.getLiveHisId())) {
                        log.info("执行个播主播离线关播检测开始 准备关闭直播:{}", JSON.toJSONString(val));
                        rtcRoomService.stopSingleRtcLive(val.getRoomId(), val.getUserId(), "2.2.0", "主播离线5分钟关播");
                        log.info("执行个播主播离线关播检测结束 关闭直播:{}", JSON.toJSONString(val));
                        rtcRoomLiveOfflineHisService.setUnDealRecord(val.getHisId());
                    } else {
                        log.info("执行个播主播离线关播检测结束 之前直播场次已结束, 不再进行处理:{}, his:{}", JSON.toJSONString(val), JSON.toJSONString(currentRound));
                        rtcRoomLiveOfflineHisService.setUnDealRecord(val.getHisId());
                    }
                }
            }
        } catch (Exception e) {
            log.error("执行个播主播离线关播检测失败", e);
            return ReturnT.FAIL;
        }

        log.info("执行个播主播离线关播检测完成");
        return ReturnT.SUCCESS;
    }

}
