package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.pojo.activity.ActivityVO;
import com.starbuds.server.common.pojo.activity.AssociatedActivityItem;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.db.StatActivityAnnaulDO;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.FamilyAuditDAO;
import com.starbuds.server.service.task.dao.StatActivityAnnualDAO;
import com.starbuds.server.service.task.dao.UserActivityScoreDetailDAO;
import com.starbuds.server.service.task.entity.FamilyAuditDTO;
import com.starbuds.server.service.task.entity.annual.StatActivityAnnualDTO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/***
 * @author qinjiang
 * @description 统计周年庆典活动
 * @date 2021/12/14
 */
@Component
@Slf4j
public class StatActivityAnnaulJob {

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    UserActivityScoreDetailDAO userActivityScoreDetailDAO;

    @Resource
    StatActivityAnnualDAO statActivityAnnualDAO;

    @Resource
    FamilyAuditDAO familyAuditDAO;

    /**
     * 统计
     * <p>
     * 获取活动类型，是否查找所有子活动ID
     * param 活动参数 不传默认统计昨日，如果传时间格式：yyyy-MM-dd 后将统计参数指定的日期
     */
    @XxlJob(value = JobHandlers.STAT_ACTIVITY_ANNUAL_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT statActivityAnnaulJob(String param) {
        log.info("-- STAT_ACTIVITY_ANNUAL_TASK --:统计服务开始");

        List<ActivityTypeEnum> activityTypeEnumList = Arrays.asList(ActivityTypeEnum.PEAKFAMILY, ActivityTypeEnum.CHARMING_ROOM, ActivityTypeEnum.INFLUENCER, ActivityTypeEnum.GORGEOUS);
        activityTypeEnumList.stream().forEach(typeEnum -> {
            long startTimestamp = System.currentTimeMillis();
            log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + typeEnum.getDesc() + " 开始");

            ActivityVO activityVO = activityService.getActivityByType(typeEnum.getValue());
            if (activityVO == null) {
                log.info("-- STAT_ACTIVITY_ANNUAL_TASK:" + typeEnum.getDesc() + ":未查找到活动");
                return;
            }

            ActivityDO activityDO = activityService.getActivity(activityVO.getActivityId());
            List<Long> activityIds = Arrays.asList(activityDO.getActivityId());
            if (typeEnum.getValue() == ActivityTypeEnum.CHARMING_ROOM.getValue()) {
                List<AssociatedActivityItem> itemList = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedActivityItem.class);
                if (CollectionUtil.isEmpty(itemList) || itemList.size() <= 0) {
                    log.info("-- STAT_ACTIVITY_ANNUAL_TASK:附加活动值为空");
                    return;
                }
                activityIds = itemList.stream().map(m -> m.getItemId()).collect(Collectors.toList());
            }

            log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + typeEnum.getDesc() + " " + JSON.toJSON(activityIds));
            statDayRank(typeEnum, activityIds, param);
            log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + typeEnum.getDesc() + " 结束  耗时：" + (System.currentTimeMillis() - startTimestamp) + "ms");
        });

        log.info("-- STAT_ACTIVITY_ANNUAL_TASK --:统计服务结束");

        return ReturnT.SUCCESS;
    }

    private void statDayRank(ActivityTypeEnum activityTypeEnum, List<Long> activityIds, String param) {
        Date yesterDayDate = DateUtil.getYesterday(new Date());
        long actionTimestamp = System.currentTimeMillis();
        //统计的时间戳，当日零点
        long statTimestamp = yesterDayDate.getTime();
        //统计时间戳，结束时间
        long statEndTimestamp = statTimestamp + 86400 * 1000;
        //统计的时间，长整型 yyyyMMdd
        long statDayInteger = DateUtil.getIntDate(yesterDayDate).longValue();

        if (StringUtils.isNotBlank(param)) {
            try {
                SimpleDateFormat sm = new SimpleDateFormat("yyyy-MM-dd");
                Date d = sm.parse(param);
                statTimestamp = d.getTime();
                statEndTimestamp = statTimestamp + 86400 * 1000;
                statDayInteger = DateUtil.getIntDate(d).longValue();
            } catch (Exception e) {
                log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + activityTypeEnum.getDesc() + " 日期参数转换失败xxxx");
                return;
            }
        }

        List<StatActivityAnnualDTO> activityAnnualDTOList = this.searchActivityAnnualDetailList(activityTypeEnum, activityIds, statDayInteger, statDayInteger, statTimestamp, statEndTimestamp);
        if (CollectionUtils.isEmpty(activityAnnualDTOList) || activityAnnualDTOList.size() <= 0) {
            log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + activityTypeEnum.getDesc() + " 没有需要统计的数据");
            return;
        }

        AtomicInteger idx = new AtomicInteger(1);
        long finalYesterTimestamp = statTimestamp;
        List<StatActivityAnnaulDO> activityAnnaulDOList = activityAnnualDTOList.stream()
                .sorted(Comparator.comparingLong(StatActivityAnnualDTO::getScore).reversed())
                .map(m -> {
                    StatActivityAnnaulDO annaulDO = new StatActivityAnnaulDO();
                    annaulDO.setHisId(identityService.getId());
                    annaulDO.setRankNo(idx.getAndIncrement());
                    annaulDO.setPartakeUnitNum(0);
                    annaulDO.setPartakeTarget(m.getTargetId());//房间ID
                    annaulDO.setPartakePeopleNum(0);
                    annaulDO.setRecordNum(m.getRecordNum());//贡献人数
                    annaulDO.setJoinFamilyNum(Optional.ofNullable(m.getJoinFamilyNum()).orElse(0));
                    annaulDO.setExitFamilyNum(Optional.ofNullable(m.getExitFamilyNum()).orElse(0));
                    annaulDO.setOverflowNum(Optional.ofNullable(m.getOverflow()).orElse(0));
                    annaulDO.setScore(m.getScore());//战力
                    annaulDO.setDimension(0);
                    annaulDO.setActivityType(activityTypeEnum.getValue());
                    annaulDO.setStatTimestamp(finalYesterTimestamp);//日时间戳
                    annaulDO.setCreateTimestamp(actionTimestamp);

                    if (activityTypeEnum.getValue() == ActivityTypeEnum.PEAKFAMILY.getValue()) {
                        annaulDO.setPartakePeopleNum(m.getRecordNum());//家族条件下为人数
                        annaulDO.setRecordNum(0);//贡献人数
                    }

                    return annaulDO;
                }).collect(Collectors.toList());

        //日排名
        statActivityAnnualDAO.deleteBatchDayStat(activityTypeEnum.getValue(), statTimestamp);
        statActivityAnnualDAO.batchInsertDayRank(activityAnnaulDOList);

        //日统计：参与人数
        Long dayPartakePeopleNum = userActivityScoreDetailDAO.getPartakePeopleNum(activityIds, statDayInteger, statDayInteger);

        StatActivityAnnaulDO dayAnnualStat = new StatActivityAnnaulDO();
        dayAnnualStat.setHisId(identityService.getId());
        dayAnnualStat.setRankNo(0);
        dayAnnualStat.setPartakeUnitNum(activityAnnaulDOList.size());
        dayAnnualStat.setPartakeTarget(0L);
        dayAnnualStat.setPartakePeopleNum(dayPartakePeopleNum.intValue());
        dayAnnualStat.setRecordNum((int) activityAnnaulDOList.stream().mapToLong(StatActivityAnnaulDO::getRecordNum).sum());
        dayAnnualStat.setJoinFamilyNum((int) activityAnnaulDOList.stream().mapToLong(StatActivityAnnaulDO::getJoinFamilyNum).sum());
        dayAnnualStat.setExitFamilyNum((int) activityAnnaulDOList.stream().mapToLong(StatActivityAnnaulDO::getExitFamilyNum).sum());
        dayAnnualStat.setOverflowNum((int) activityAnnaulDOList.stream().mapToLong(StatActivityAnnaulDO::getOverflowNum).sum());
        dayAnnualStat.setScore(activityAnnaulDOList.stream().mapToLong(StatActivityAnnaulDO::getScore).sum());
        dayAnnualStat.setDimension(1);
        dayAnnualStat.setActivityType(activityTypeEnum.getValue());
        dayAnnualStat.setStatTimestamp(statTimestamp);
        dayAnnualStat.setCreateTimestamp(actionTimestamp);
        statActivityAnnualDAO.batchInsertDayRank(Arrays.asList(dayAnnualStat));
    }


    /**
     * 查找周年庆活动详细列表数据
     *
     * @param activityTypeEnum
     * @param activityIds
     * @param dayBegin
     * @param dayEnd
     * @return
     */
    private List<StatActivityAnnualDTO> searchActivityAnnualDetailList(ActivityTypeEnum activityTypeEnum, List<Long> activityIds, Long dayBegin, Long dayEnd, Long startTimestamp, Long endTimestamp) {
        int pageIndex = 1;
        int pageSize = 300;
        boolean isRunloop = true;

        //处理好的目标数据，插入数据库对象列表
        List<StatActivityAnnualDTO> activityAnnualDTOList = new ArrayList<>();

        while (isRunloop) {
            List<StatActivityAnnualDTO> searchActivityAnnualList = new ArrayList<>();
            if (activityTypeEnum.getValue() == ActivityTypeEnum.PEAKFAMILY.getValue()) {
                searchActivityAnnualList = userActivityScoreDetailDAO.getStatActivityFamilyList(activityIds, dayBegin, dayEnd, (pageIndex - 1) * pageSize, pageSize);
            } else if (activityTypeEnum.getValue() == ActivityTypeEnum.CHARMING_ROOM.getValue()) {
                searchActivityAnnualList = userActivityScoreDetailDAO.getStatActivityRoomList(activityIds, dayBegin, dayEnd, (pageIndex - 1) * pageSize, pageSize);
            } else {
                searchActivityAnnualList = userActivityScoreDetailDAO.getStatActivityUserList(activityIds, dayBegin, dayEnd, (pageIndex - 1) * pageSize, pageSize);
            }

            if (CollectionUtils.isEmpty(searchActivityAnnualList) || searchActivityAnnualList.size() <= 0) {
                break;
            }
            if (searchActivityAnnualList.size() < pageSize) {
                isRunloop = false;
            }
            searchActivityAnnualList.removeIf(m -> m.getTargetId() == null);
            activityAnnualDTOList.addAll(searchActivityAnnualList);
            pageIndex++;
        }

        //不是巅峰家族直接返回数据，是巅峰家族需要额外对家族相关数据进行匹配赋值
        if (activityTypeEnum.getValue() != ActivityTypeEnum.PEAKFAMILY.getValue()) {
            return activityAnnualDTOList;
        }

        List<Long> familyIds = activityAnnualDTOList.stream().map(m -> m.getTargetId()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(familyIds) || familyIds.size() <= 0) {
            log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + activityTypeEnum.getDesc() + " 没有查找到参与的家族");
            return new ArrayList<StatActivityAnnualDTO>();
        }
        log.info("-- STAT_ACTIVITY_ANNUAL_TASK 活动:" + activityTypeEnum.getDesc() + " 活动id列表：" + JSON.toJSONString(familyIds));

        Map<String, FamilyAuditDTO> familyAuditDTOMap = familyAuditDAO.getFamilyTurnover(familyIds, startTimestamp, endTimestamp);
        activityAnnualDTOList.stream().forEach(m -> {
            String joinKey = String.join("_", m.getTargetId().toString(), String.valueOf(ApplyTypeEnum.JOIN.getCode()));
            String exitKey = String.join("_", m.getTargetId().toString(), String.valueOf(ApplyTypeEnum.LEAVE.getCode()));

            FamilyAuditDTO joinAudit = familyAuditDTOMap.get(joinKey);
            if (joinAudit != null) {
                m.setJoinFamilyNum(joinAudit.getCount());
            }
            FamilyAuditDTO exitAudit = familyAuditDTOMap.get(exitKey);
            if (exitAudit != null) {
                m.setExitFamilyNum(exitAudit.getCount());
            }
        });

        return activityAnnualDTOList;
    }

}
