package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.service.statistics.StatRiskService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.util.DingDingUtil;
import com.starbuds.server.service.task.util.SpringContextUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @Author lujiao
 * @DateTime 2021/7/8 15:35
 * @Description
 */
@Slf4j
@Component
public class StatRiskManageJob {

    @Reference(version = "1.0.0")
    StatRiskService statRiskService;

    @Resource
    DingDingUtil dingDingUtil;

    private static String WARN_TEXT = "风控数据有异常, 请及时处理";

    @XxlJob(value = JobHandlers.STAT_RISK_HISTORY_TASK)
    public ReturnT<String> statRiskHistoryHandler(String param) {
        log.info("执行历史数据生成 {}", param);
        long curTime = System.currentTimeMillis();
        try {
            statRiskService.generateHistoryRiskData();
        } catch (Exception e) {
            log.error("生成风控历史数据执行出错", e);
            return ReturnT.FAIL;
        }
        long endTime = System.currentTimeMillis();
        log.info("执行成功,共耗时{}s", (endTime - curTime) / 1000);
        return ReturnT.SUCCESS;
    }

    @XxlJob(value = JobHandlers.STAT_GENERATE_RISK_DATA_TASK)
    public ReturnT<String> statGenerateRiskDataHandler(String param) {
        log.info("执行风控统计数据生成 {}", param);
        long curTime = System.currentTimeMillis();
        try {
            Date yesterday = DateUtil.getYesterday(new Date());
            List<String> riskList = statRiskService.generateDayRiskData(yesterday);
            if (CollectionUtil.isNotEmpty(riskList)) {
                // 这里进行钉钉推送
                String evn = "生产环境 : ";
                if (SpringContextUtil.isDev()) {
                    evn = "测试环境 : ";
                }
                if (!SpringContextUtil.isDev()) {
                    StringBuilder stringBuilder = new StringBuilder();
                    stringBuilder.append(evn + WARN_TEXT + "\n");
                    riskList.forEach(i -> {
                        stringBuilder.append(i + "\n");
                    });
                    dingDingUtil.sendDingDingWarn(stringBuilder.toString());
                }
            }
            log.info("执行风控数据日数据生成完成", param);
            statRiskService.generateWeekRiskData(yesterday);
            log.info("执行风控数据周数据生成完成", param);
            statRiskService.generateMonthRiskData(yesterday);
            log.info("执行风控数据月数据生成完成", param);
        } catch (Exception e) {
            log.error("生成风控数据执行出错", e);
            return ReturnT.FAIL;
        }
        long endTime = System.currentTimeMillis();
        log.info("执行成功,共耗时{}s", (endTime - curTime) / 1000);
        return ReturnT.SUCCESS;
    }
}
