package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.format.FastDateFormat;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.*;

/***
 * @author qinjiang
 * @description 统计房间弹幕任务
 * @date 2021/9/25
 */
@Slf4j
@Component
public class StatRoomBarrageJob {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @XxlJob(value = JobHandlers.STAT_ROOM_BARRAGE_TASK)
    public ReturnT<String> statRoomBarrageHandler(String params) {
        log.info("statRoomBarrageHandler");

//        DateUtil.yesterday()
        /*  DateTime date = YESTERDAY == type ? DateUtil.yesterday() : new DateTime();  */

        //拿到目标时间
        String strToday = FastDateFormat.getInstance("dd").format(new DateTime());
        //目标时间获取所有的房间
        String allRoomKey = RedisCacheKeys.getLiveBarrageDayKey(strToday, 0L);
        Set<Tuple> tupleSet = cacheService.zReverseRangeWithScores(allRoomKey, 0L, -1);
        if (CollectionUtil.isEmpty(tupleSet)) {
            return ReturnT.SUCCESS;
        }

        List<String> listRoomID = new ArrayList<>(tupleSet.size());
        tupleSet.stream().forEach(tuple -> {
            String strRoomID = tuple.getElement();
            String barrageRoomKey = RedisCacheKeys.getLiveBarrageDayKey(strToday, Long.valueOf(strRoomID));
            Set<Tuple> tupleBarrageSet = cacheService.zReverseRangeWithScores(barrageRoomKey, 0L, -1);

            int barrageNum = tuple.getScore().intValue();
            int peopleNum = tupleBarrageSet.size();

            log.info("--statRoomBarrageHandler  roomID:" + strRoomID + "   peopleNum:" + peopleNum + "   barrageNum:" + barrageNum);
        });

        return ReturnT.SUCCESS;
    }
}
