package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.pojo.db.FollowRelationDO;
import com.starbuds.server.common.pojo.db.UserRelationStatsDO;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UFollowRelationDAO;
import com.starbuds.server.service.task.dao.URelationStatDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class StatUserRelationJob {

    @Autowired
    URelationStatDAO uRelationStatDAO;

    @Autowired
    UFollowRelationDAO uFollowRelationDAO;

    @XxlJob(value = JobHandlers.STAT_USER_RELATION_TASK)
    public ReturnT<String> statUserRelationHandler(String param) {
        log.info("开始生成历史数据");
        buildStatUserRelation();

        return ReturnT.SUCCESS;
    }

    public void buildStatUserRelation() {
        // 获取最新的一条数据, 以此为基准时间
        Long begin = System.currentTimeMillis();
        Long endTime = System.currentTimeMillis();
        log.info("[buildStatUserRelation]:endTime:" + endTime);
        int page = 1;
        int pageSize = 10000;

        while (true) {
            List<FollowRelationDO> followHisList = uFollowRelationDAO.getFollowHis(endTime, (page - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(followHisList) || followHisList.size() <= 0) {
                break;
            }

            Map<Long, UserRelationStatsDO> batchStatsDoMap = new HashMap<>();

            int followCount = followHisList.size();
            for (int index = 0; index < followCount; index++) {
                FollowRelationDO loopDO = followHisList.get(index);

                if (loopDO.getStatus() <= 0) {
                    continue;
                }
                Long userID = loopDO.getUserId();
                Long targetID = loopDO.getTargetId();
                Integer direction = loopDO.getRelationDirection();//关系方向 -1: B -> A   0: A<->B    1 A->B

                UserRelationStatsDO userStatsDO;
                UserRelationStatsDO targetStatsDO;
                if (batchStatsDoMap.get(userID) == null) {
                    userStatsDO = new UserRelationStatsDO();
                    userStatsDO.setUserId(userID);
                    userStatsDO.setFansCount(0L);
                    userStatsDO.setFollowCount(0L);
                } else {
                    userStatsDO = batchStatsDoMap.get(userID);
                }

                if (batchStatsDoMap.get(targetID) == null) {
                    targetStatsDO = new UserRelationStatsDO();
                    targetStatsDO.setUserId(targetID);
                    targetStatsDO.setFansCount(0L);
                    targetStatsDO.setFollowCount(0L);
                } else {
                    targetStatsDO = batchStatsDoMap.get(targetID);
                }

                if (direction == -1) {
                    userStatsDO.setFansCount(userStatsDO.getFansCount() + 1L);
                    targetStatsDO.setFollowCount(targetStatsDO.getFollowCount() + 1L);
                } else if (direction == 0) {
                    userStatsDO.setFansCount(userStatsDO.getFansCount() + 1L);
                    userStatsDO.setFollowCount(userStatsDO.getFollowCount() + 1L);
                    targetStatsDO.setFansCount(targetStatsDO.getFansCount() + 1L);
                    targetStatsDO.setFollowCount(targetStatsDO.getFollowCount() + 1L);
                } else if (direction == 1) {
                    userStatsDO.setFollowCount(userStatsDO.getFollowCount() + 1L);
                    targetStatsDO.setFansCount(targetStatsDO.getFansCount() + 1L);
                }

                batchStatsDoMap.put(userID, userStatsDO);
                batchStatsDoMap.put(targetID, targetStatsDO);
            }

            List<String> batchStatsSQLs = new ArrayList<>();
            List<UserRelationStatsDO> batchStatsDoList = new ArrayList<>(batchStatsDoMap.values());

            batchStatsDoList.forEach(statsDO -> {
                String sqlFormat = "INSERT INTO tb_user_relation_stats(userId,fansCount,followCount) values (%d,%d,%d) on duplicate key update fansCount=fansCount + %d, followCount = followCount + %d";
                String sql = String.format(sqlFormat, statsDO.getUserId(), statsDO.getFansCount(), statsDO.getFollowCount(), statsDO.getFansCount(), statsDO.getFollowCount());
                batchStatsSQLs.add(sql);
            });

            uRelationStatDAO.insertOrUpdateBatch(batchStatsSQLs);

            page++;
        }

        Long end = System.currentTimeMillis();
        log.info("[buildStatUserRelation]数据执行共耗时 :" + (end - begin) + "毫秒");
    }


}
