package com.starbuds.server.service.task.job;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.FeedDO;
import com.starbuds.server.common.pojo.db.FeedPhotoWallDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.FeedPhotoWallDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;

import static com.starbuds.server.common.pojo.util.PageListUtil.nextPage;

/**
 * 同步照片墙（目前只使用一次）
 *
 * @author aoe
 * @date 2021/2/1
 */
@Slf4j
@Component
@Deprecated
public class SyncPhotoWallJob {

    @Resource
    private FeedPhotoWallDAO feedPhotoWallDAO;

    @Resource
    private UserDAO userDAO;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @XxlJob(value = JobHandlers.SYNC_PHOTO_WALL)
    public ReturnT<String> syncPhotoWall(String param) {
        log.info("同步照片墙开始");
        int count = doSyncPhotoWall(param);
        log.info("同步照片墙结束，共 {} 条", count);
        ReturnT<String> returnT = ReturnT.SUCCESS;
        returnT.setContent("同步数据 " + count + " 条");
        return returnT;
    }

    private int doSyncPhotoWall(String param) {
        Integer pageIndex = 1;
        // PageList 中最大为 50
        int pageSize = 50;

        if (StringUtils.isNotBlank(param) && param.contains(",")) {
            String[] array = param.split(",");
            if (array.length == 2) {
                int v1 = NumberUtils.toInt(array[0], -1);
                int v2 = NumberUtils.toInt(array[1], -1);
                if (v1 > 0 && v2 > 0) {
                    pageIndex = v1;
                    pageSize = v2;
                }
            }
        }
        log.info("同步照片墙 pageIndex = {} pageSize = {}", pageIndex, pageSize);

        AtomicInteger count = new AtomicInteger();
        while (pageIndex != null) {
            PageList<FeedDO> pageList = getFeedPageList(pageIndex, pageSize);
            List<FeedDO> feedList = pageList.getList();
            log.info("同步照片墙 feedList size = {} ", feedList.size());
            if (CollectionUtils.isEmpty(feedList)) {
                break;
            }

            List<FeedPhotoWallDO> insertList = new ArrayList<>();
            feedList.stream()
                    .filter(Objects::nonNull)
                    .filter(f -> f.getUserId() != null)
                    .filter(f -> StringUtils.isNotBlank(f.getUrl()))
                    .filter(f -> !f.getUrl().contains(","))
                    .forEach(feed -> {
                        Long userId = feed.getUserId();
                        String url = feed.getUrl();
                        if (feedPhotoWallDAO.selectPhotoWallCount(userId, url) > 0) {
                            // 已经同步过数据
                            return;
                        }
                        FeedPhotoWallDO po = createPhotoWall(userId, url);
                        insertList.add(po);
                    });
            int insertCount = feedPhotoWallDAO.insertBatch(insertList);
            log.info("同步照片墙 第 {} 页 {} 条", pageIndex, insertCount);
            count.addAndGet(insertCount);
            pageIndex = nextPage(pageList);
        }
        return count.get();
    }

    private PageList<FeedDO> getFeedPageList(int pageNum, int pageSize) {
        PageList<FeedDO> page = new PageList<>(pageNum, pageSize);
        page.setList(feedPhotoWallDAO.selectFeedList(page.getPageCursor(), pageSize));
        page.setTotal(feedPhotoWallDAO.selectFeedListCount());
        return page;
    }

    /**
     * 创建照片墙
     *
     * @param userId
     * @param url
     * @return
     */
    private FeedPhotoWallDO createPhotoWall(Long userId, String url) {
        FeedPhotoWallDO fpw = new FeedPhotoWallDO();
        fpw.setUserId(userId);
        fpw.setUrl(url);
        fpw.setSort(0);
        fpw.setIsDeleted(BoolIntVal.False);

        Long id = identityService.getId();
        fpw.setFeedPhotoWallId(id);
        fpw.setStatus(FeedStatusEnum.PASSED.value());
        fpw.setAdminId(0L);
        fpw.setVersion(0L);
        fpw.setCreateTime(System.currentTimeMillis());
        fpw.setUpdateTime(fpw.getCreateTime());

        return fpw;
    }
}
