package com.starbuds.server.service.task.job;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.RefTypeEnum;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.constant.finance.WalletWithdrawHisApplyStatus;
import com.starbuds.server.common.pojo.db.UserAuthDO;
import com.starbuds.server.common.pojo.db.UserIdCardDO;
import com.starbuds.server.common.pojo.db.UserWalletDO;
import com.starbuds.server.common.pojo.db.UserWalletWithdrawHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UserSignContractDAO;
import com.starbuds.server.service.task.dao.UserWalletDAO;
import com.starbuds.server.service.task.dao.UserWalletWithdrawHisDAO;
import com.starbuds.server.service.task.entity.sailtob.SailtobConfig;
import com.starbuds.server.service.task.entity.sailtob.SailtobTax;
import com.starbuds.server.service.task.entity.user.UserSignContractDO;
import com.starbuds.server.service.task.impl.sailtob.SailtobApiService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/***
 * @author qinjiang
 * @description 用户签约合同
 * @date 2021/12/1
 */
@Slf4j
@Component
public class UserSignContractJob {
    private static final String LogPrefix = "【智企行】用户签名服务：";

//    private SailtobConfig sailtobConfig = null;

    @Resource
    UserWalletWithdrawHisDAO userWalletWithdrawHisDAO;

    @Resource
    UserWalletDAO userWalletDAO;

    @Resource
    UserSignContractDAO userSignContractDAO;

    @Resource
    SailtobApiService sailtobApiService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    UserService userService;

//    @Reference(version = "1.0.0")
//    SailtobService sailtobService;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @XxlJob(value = JobHandlers.USER_SAILTOB_SIGN_CONTRACT_TASK)
    public ReturnT<String> sailtobSignContractHandler(String param) {
        SailtobConfig sailtobConfig = getSailtobConfig();

        if (sailtobConfig == null) {
            log.info("获取智企行配置为空");
            return ReturnT.FAIL;
        }

        List<SailtobTax> taxList = sailtobConfig.getTax();
        if (CollectionUtils.isEmpty(taxList) || taxList.size() <= 0) {
            log.info("获取智企行配置的税地列表为空");
            return ReturnT.FAIL;
        }

        int pageSize = 100;
        int pageIndex = 1;
        boolean isRunloop = true;
        while (isRunloop) {
            List<UserWalletWithdrawHisDO> withdrawHisDOList = userWalletWithdrawHisDAO.getUserWalletWithdrawList(
                    (pageIndex - 1) * pageSize,
                    pageSize,
                    WalletWithdrawHisApplyStatus.WAIT.getValue(),
                    RefTypeEnum.USER_WITHDRAW.getValue());
            if (CollectionUtils.isEmpty(withdrawHisDOList) || withdrawHisDOList.size() <= 0) {
                break;
            }
            if (withdrawHisDOList.size() < pageSize) {
                isRunloop = false;
            }

            for (UserWalletWithdrawHisDO hisDO : withdrawHisDOList) {
                UserWalletDO userWallet = userWalletDAO.selectById(hisDO.getWalletId());
                UserAuthDO userAuth = userService.getUserAuth(userWallet.getUserId());
                Long userId = userWallet.getUserId();
                String phone = userService.getPhone(userId);
                String cardNo = hisDO.getCardNo();
                String idCard = userAuth.getIdCardNo();

                for (SailtobTax tax : taxList) {
                    Long payType = Long.valueOf(tax.getId());

//                    UserSignContractDO userSignContractDO = userSignContractDAO.selectUserSignRecord(userId, ThirdPartyCompanyEnum.SAILTOB.getId(),Long.valueOf(tax.getLevyCode()));
//                    if (userSignContractDO != null) {
//                        continue;
//                    }
//
                    UserIdCardDO userIdCardDO = userSignContractDAO.getUserIDCardByUserId(userId);
                    if (userIdCardDO == null) {
                        log.info("智企行：提现请求用户签约查找不成功");
                        continue;
                    }
                    //获取用户身份证
                    String idCardFront = userIdCardDO.getFaceImg();
                    String idCardBack = userIdCardDO.getNationImg();
                    boolean isUserSign = sailtobApiService.isUserSign(userId, payType);
                    if (!isUserSign) {
                        boolean signResult = sailtobApiService.signSoho(userAuth.getRealName(), cardNo, phone, idCard, idCardFront, idCardBack, userId + "", payType);
                        if (!signResult) {
                            log.info("{}签名服务不成功：userId:{} tax:{}", LogPrefix, userId, tax.getLevyCode());
                            continue;
                        }

                        UserSignContractDO signContractDO = new UserSignContractDO();
                        signContractDO.setHisId(identityService.getId());
                        signContractDO.setUserId(userId);
                        signContractDO.setPlatfrom(ThirdPartyCompanyEnum.SAILTOB.getId());
                        signContractDO.setFlagId(Long.valueOf(tax.getLevyCode()));
                        signContractDO.setName(userAuth.getRealName());
                        signContractDO.setBankCardNo(cardNo);
                        signContractDO.setIdCardNo(idCard);
                        signContractDO.setIdCardFront(idCardFront);
                        signContractDO.setIdCardBack(idCardBack);
                        signContractDO.setPhone(phone);
                        signContractDO.setStatus(1);
                        signContractDO.setExpireTimestamp(0L);
                        signContractDO.setTimestamp(System.currentTimeMillis());

                        userSignContractDAO.createUserSign(signContractDO);
                    }
                }
            }

            pageIndex++;
        }

        return ReturnT.SUCCESS;
    }

    /**
     * 初始智企行配置
     *
     * @throws Exception
     */
    private SailtobConfig getSailtobConfig() {
        String strSailtobConfig = releaseConfigService.getCachReleaseConfig("sailtob");
        if (StringUtils.isEmpty(strSailtobConfig)) {
            log.info("获取在线打款智企行配置失败");
            return null;
        }

        return JSON.parseObject(strSailtobConfig, SailtobConfig.class);
    }
}
