package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.statistics.StatCommonTypeEnum;
import com.starbuds.server.common.pojo.data.Wallet;
import com.starbuds.server.common.pojo.db.ChannelWalletDO;
import com.starbuds.server.common.pojo.db.StatCommonDO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ChannelWalletDAO;
import com.starbuds.server.service.task.dao.FamilyWalletDAO;
import com.starbuds.server.service.task.dao.StatCommonDAO;
import com.starbuds.server.service.task.dao.UserWalletDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/2 15:38
 * @Description 用户背包
 */
@Slf4j
@Component
public class UserWalletJob {

    /**
     * 分页大小
     */
    private static final int pageSize = 500;

    /**
     * 钱包类型
     */
    private static final List<Integer> walletTypes = Arrays.asList(
            WalletTypeEnum.STAR_DIAMOND.getValue(),
            WalletTypeEnum.STAR_COIN.getValue(),
            WalletTypeEnum.STAR_DUST.getValue(),
            WalletTypeEnum.PINK_DIAMOND.getValue(),
            WalletTypeEnum.STAR_YUAN.getValue()
    );


    @Resource
    UserWalletDAO userWalletDAO;

    @Resource
    StatCommonDAO statCommonDAO;

    @Resource
    ChannelWalletDAO channelWalletDAO;

    @Resource
    FamilyWalletDAO familyWalletDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.USER_WALLET)
    public ReturnT<String> userWalletHandler(String param) {
        log.info("执行用户钱包存量价值 {}", param);

        long curTime = System.currentTimeMillis();
        String curDate = DateUtil.format(new Date(curTime), "yyyyMMdd");

        for (int i = 0; i < walletTypes.size(); i++) {
            Integer walletType = walletTypes.get(i);
            WalletTypeEnum walletTypeEnum = WalletTypeEnum.toEnum(walletType);
            // 总条数
            int total = userWalletDAO.countUserWallets(walletType);
            // 确定页大小
            int pageCount = total % pageSize == 0 ? total / pageSize : total / pageSize + 1;
            log.info("总条数{} 页大小{}", total, pageCount);
            // 分页处理
            for (int j = 0; j < pageCount; j++) {
                int pageCursor = j * pageSize;
                List<Wallet> wallets = userWalletDAO.selectUserWalletPageList(walletType, pageCursor, pageSize);
                log.info("当前正在处理{}钱包 第{}页 ", walletTypeEnum.getName(), j);
                if (CollectionUtil.isNotEmpty(wallets)) {
                    List<StatCommonDO> list = new ArrayList<>();
                    Set<Long> channelIds = wallets.stream().map(Wallet::getChannelId).collect(Collectors.toSet());
                    for (Long channelId : channelIds) {
                        StatCommonDO commonDO = new StatCommonDO();
                        switch (walletTypeEnum) {
                            case STAR_DIAMOND:
                                commonDO.setStatType(StatCommonTypeEnum.USER_WALLET_BALANCE.getValue());
                                break;
                            case STAR_COIN:
                                commonDO.setStatType(StatCommonTypeEnum.USER_WALLET_BALANCE_COIN.getValue());
                                break;
                            case STAR_DUST:
                                commonDO.setStatType(StatCommonTypeEnum.USER_WALLET_BALANCE_DUST.getValue());
                                break;
                            case PINK_DIAMOND:
                                commonDO.setStatType(StatCommonTypeEnum.USER_WALLET_BALANCE_PINK.getValue());
                                break;
                            case STAR_YUAN:
                                commonDO.setStatType(StatCommonTypeEnum.USER_WALLET_BALANCE_YUAN.getValue());
                                break;
                            default:
                                break;
                        }
                        if (commonDO.getStatType() == null) {
                            break;
                        }
                        commonDO.setChannelId(channelId);
                        commonDO.setCreateTime(curTime);
                        commonDO.setStatDate(Convert.toLong(curDate));
                        List<Wallet> filterList = wallets.stream().filter(e -> channelId.equals(e.getChannelId())).collect(Collectors.toList());
                        commonDO.setStatValue(Convert.toBigDecimal(filterList.stream().map(Wallet::getBalance).collect(Collectors.summarizingLong(Long::longValue)).getSum()));
                        list.add(commonDO);
                    }
                    statCommonDAO.batchInsert(list);
                    log.info("{}钱包保存成功", walletTypeEnum.getName());
                }
            }
        }

        // 渠道星钻钱包
        List<ChannelWalletDO> channelWallets = channelWalletDAO.selectWallets(WalletTypeEnum.STAR_DIAMOND.getValue());
        if (CollectionUtil.isNotEmpty(channelWallets)) {
            List<StatCommonDO> list = new ArrayList<>();
            for (ChannelWalletDO item : channelWallets) {
                StatCommonDO commonDO = new StatCommonDO();
                commonDO.setChannelId(item.getChannelId());
                commonDO.setStatType(StatCommonTypeEnum.CHANNEL_WALLET_STAR_DIAMOND.getValue());
                commonDO.setCreateTime(curTime);
                commonDO.setStatDate(Convert.toLong(curDate));
                commonDO.setStatValue(Convert.toBigDecimal(item.getBalance()));
                list.add(commonDO);
            }

            statCommonDAO.batchInsert(list);
            log.info("渠道钱包保存成功");
        }

        // 家族钱包粉钻
        BigDecimal bigDecimal = familyWalletDAO.selectSumWallet();
        StatCommonDO commonDO = new StatCommonDO();
        commonDO.setChannelId(0L);
        commonDO.setCreateTime(curTime);
        commonDO.setStatValue(bigDecimal);
        commonDO.setStatDate(Convert.toLong(curDate));
        commonDO.setStatType(StatCommonTypeEnum.FAMILY_WALLET_STAR_DIAMOND.getValue());
        statCommonDAO.batchInsert(Collections.singletonList(commonDO));

        log.info("执行成功");
        return ReturnT.SUCCESS;
    }

}
