package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UserVipDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * 会员定时器
 *
 * @author Lay
 * @date 2020/4/4
 */
@Component
@Slf4j
public class VipJob {

    @Resource
    UserVipDAO userVipDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.VIP_EXPIRE)
    public ReturnT<String> vipExpireJobHandler(String param) {
        log.info("触发会员过期处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 查询日期处理
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer day;
        if (StringUtils.isNotBlank(param)) {
            // 有传参取传参的
            day = Convert.toInt(param);
        } else {
            // 不然就是今天的
            Calendar calendar = Calendar.getInstance();
            day = Convert.toInt(sdf.format(calendar.getTime()));
        }

        if (day != null) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            long todayTime = calendar.getTimeInMillis();

            // 随机获取一台来更新
            int index = RandomUtil.randomInt(0, shardingVO.getTotal());
            if (index == shardingVO.getIndex()) {
                userVipDAO.updateExpired(todayTime, CommonStatus.Disabled);
            }
        }
        return ReturnT.SUCCESS;
    }
}
