package com.starbuds.server.service.task.provider;

import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatPlatformDO;
import com.starbuds.server.service.task.dao.StatPlatformDAO;
import com.starbuds.server.service.task.entity.stat.StatDay;
import com.starbuds.server.service.task.entity.stat.StatDayPretty;
import com.starbuds.server.service.task.util.StatUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

import static java.util.stream.Collectors.toList;

/**
 * 平台统计
 *
 * @author aoe
 * @date 2021/9/7
 */
@Slf4j
@Component
public class StatPlatformProvider {

    @Resource
    private StatPlatformDAO statPlatformDAO;

    /**
     * 获得一天的统计数据
     *
     * @param date
     * @param statPlatformType
     * @return
     */
    public StatDayPretty getStatByYearMonthDay(Long date, StatPlatformTypeEnum statPlatformType) {
        Integer statType = statPlatformType.getValue();
        int statTimeType = StatTimeType.YEAR_MONTH_DAY;
        StatPlatformDO statPlat = statPlatformDAO.selectStatByTime(date, statType, statTimeType);
        if (statPlat == null) {
            statPlat = new StatPlatformDO();
            statPlat.setStatType(statType);
            statPlat.setStatTimeType(statTimeType);
            statPlat.setStatTime(date);
            statPlat.setChannelId(0L);
            statPlat.setStatValue(BigDecimal.ZERO);
        }

        StatDay stat = StatUtil.toStatDay(statPlat);
        return StatUtil.pretty(stat);
    }

    /**
     * 返回以天为单位的统计数据
     *
     * @param startDate            yyyyMMdd
     * @param endDate              yyyyMMdd
     * @param statPlatformTypeEnum
     * @return
     */
    public List<StatDayPretty> getStatPrettyListByDay(Long startDate, Long endDate, StatPlatformTypeEnum statPlatformTypeEnum) {
        List<StatDay> list = getStatListByDay(startDate, endDate, statPlatformTypeEnum);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return StatUtil.statListByDay2Pretty(startDate, endDate, list);
    }

    /**
     * 返回以天为单位的统计数据
     *
     * @param startDate            yyyyMMdd
     * @param endDate              yyyyMMdd
     * @param statPlatformTypeEnum
     * @return
     */
    private List<StatDay> getStatListByDay(Long startDate, Long endDate, StatPlatformTypeEnum statPlatformTypeEnum) {
        List<StatPlatformDO> list = statPlatformDAO.selectStatListByTime(startDate, endDate,
                statPlatformTypeEnum.getValue(), StatTimeType.YEAR_MONTH_DAY);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list.stream()
                .map(StatUtil::toStatDay)
                .filter(Objects::nonNull)
                .collect(toList());
    }
}
