package com.starbuds.server.service.task.util;

import com.starbuds.server.common.pojo.db.StatPlatformDO;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.entity.stat.StatDay;
import com.starbuds.server.service.task.entity.stat.StatDayPretty;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 统计工具
 *
 * @author aoe
 * @date 2021/9/7
 */
public class StatUtil {

    /**
     * 转换成以天为单位的统计对象
     *
     * @param statPlat
     * @return null:数据有误
     */
    public static StatDay toStatDay(StatPlatformDO statPlat) {
        if (statPlat == null || statPlat.getStatTime() == null || statPlat.getStatValue() == null) {
            return null;
        }
        StatDay sh = new StatDay();
        sh.setStatTime(statPlat.getStatTime());
        sh.setDay("" + statPlat.getStatTime());
        sh.setValue(statPlat.getStatValue().longValue());

        return sh;
    }

    /**
     * 格式化以天为单位的统计数据
     *
     * @param startDate yyyyMMdd
     * @param endDate   yyyyMMdd
     * @param statList
     * @return
     */
    public static List<StatDayPretty> statListByDay2Pretty(Long startDate, Long endDate, List<StatDay> statList) {
        Map<String, Long> statDayMap = new HashMap<>();
        if (statList != null) {
            statList.forEach(statDay -> statDayMap.put(statDay.getDay(), statDay.getValue()));
        }

        List<StatDayPretty> result = new ArrayList<>();
        LocalDate startD = DateUtil.toLocalDate(startDate);
        LocalDate endD = DateUtil.toLocalDate(endDate);
        while (startD.compareTo(endD) < 1) {
            StatDayPretty day = new StatDayPretty();
            day.setDay(startD.format(DateTimeFormatter.BASIC_ISO_DATE));
            Long count = statDayMap.get(day.getDay());
            count = count == null ? 0 : count;
            day.setValue(count);

            result.add(day);
            startD = startD.plusDays(1);
        }

        return result;
    }

    /**
     * 格式化数据
     *
     * @param statDay
     * @return
     */
    public static StatDayPretty pretty(StatDay statDay) {
        StatDayPretty pretty = new StatDayPretty();
        LocalDate startD = DateUtil.toLocalDate(statDay.getStatTime());
        pretty.setDay(startD.format(DateTimeFormatter.BASIC_ISO_DATE));
        Long count = statDay.getValue();
        count = count == null ? 0 : count;
        pretty.setValue(count);
        return pretty;
    }
}
